program main
  use starpu_f_mod
  use iso_c_binding
  implicit none

  interface
     subroutine insert_mytask_c(a_handle, b_handle, multiplier, ctx) bind(C)
       use iso_c_binding
       type(c_ptr)           :: a_handle, b_handle
       real(c_float), value  :: multiplier
       integer(c_int), value :: ctx
     end subroutine insert_mytask_c
  end interface
     
  type(c_ptr)                 :: ha, hb, hc
  integer(c_int)              :: ncpus, nworkers,  ctx1, ctx2
  integer                     :: ret, i, j
  integer(c_int)              :: worker_ids(8)
  real, allocatable           :: a(:), b(:,:), c(:,:)
  
  ret = starpu_f_init(c_null_ptr)
  nworkers = starpu_f_worker_get_count()
  write(*,'("Starpu inited (",i1,").  Has ",i2," CPU workers")')ret, nworkers

  do i=1, nworkers
     worker_ids(i) = i-1
  end do

  ctx1 = starpu_f_sched_ctx_create(worker_ids, nworkers, f_to_c_string("ctx1"))
  ctx2 = starpu_f_sched_ctx_create(worker_ids, nworkers, f_to_c_string("ctx2"))

  call starpu_f_sched_ctx_display_workers(ctx1)
  call starpu_f_sched_ctx_display_workers(ctx2)
  
  call starpu_f_void_data_register(ha)

  call starpu_f_data_unregister(ha)
  ha = c_null_ptr
  allocate(a(5), b(5,5), c(5,5))
  a = 2.e0
  b = 0.e0
  c = 1.e0

  call starpu_f_fxt_start_profiling()
  
  call starpu_f_matrix_data_register(ha, 0, a)
  call starpu_f_matrix_data_register(hb, 0, b)
  call starpu_f_matrix_data_register(hc, 0, c)

  ! call starpu_f_sched_ctx_set_context(ctx1)
  call insert_mytask_c(ha, hb, 1.e0, ctx1)
  call insert_mytask_c(ha, hc, 1.e0, ctx1)
  call insert_mytask_c(hc, ha, 1.e0, ctx1)
  call insert_mytask_c(ha, hb, 1.e0, ctx1)

  ! call starpu_f_sched_ctx_set_context(ctx2)
  call insert_mytask_c(ha, hb, 2.e0, ctx2)
  call insert_mytask_c(ha, hc, 2.e0, ctx2)
  call insert_mytask_c(hc, ha, 2.e0, ctx2)
  call insert_mytask_c(ha, hb, 2.e0, ctx2)

  
  call starpu_f_task_wait_for_all_in_ctx(ctx1)
  write(*,'("Yuppi, all the tasks in ctx",i1," ave finished!")')ctx1

  call starpu_f_task_wait_for_all_in_ctx(ctx2)
  write(*,'("Yuppi, all the tasks in ctx",i1," ave finished!")')ctx2
  write(*,'(" ")')
  
  call starpu_f_data_acquire_read(ha)
  call starpu_f_data_acquire_read(hb)
  call starpu_f_data_acquire_read(hc)

  write(*,'("I am so appy I have acquired all the data!")')
  
  call starpu_f_data_release(ha)
  call starpu_f_data_release(hb)
  call starpu_f_data_release(hc)
  
  call starpu_f_task_wait_for_all()

  call starpu_f_sched_ctx_delete(ctx1)
  call starpu_f_sched_ctx_delete(ctx2)

  call starpu_f_data_unregister(ha)
  call starpu_f_data_unregister(hb)
  call starpu_f_data_unregister(hc)

  call starpu_f_fxt_stop_profiling()
  call starpu_f_shutdown()

  do i=1, size(b,1)
     do j=1, size(b,2)
        write(*,'(f7.3,2x)',advance='no')b(i,j)
     end do
     write(*,'(" ")')
  end do
  
  stop

contains

  function f_to_c_string(strin) result(strout)
    use iso_c_binding
    character(len=*) :: strin
    character        :: strout(len(strin)+1)
    integer          :: i

    i = len(strin)
    strout(1:i+1) = transfer(strin(1:i)//c_null_char, strout(1:i+1))
    
    return
  end function f_to_c_string
  
end program main



subroutine mytask_cpu_func(buffers, cl_arg) bind(C)
  use starpu_f_mod
  use iso_c_binding
  implicit none

  interface
     subroutine unpack_mytask_args(cl_arg, multiplier) bind(c)
       use iso_c_binding
       type(c_ptr), value :: cl_arg, multiplier
     end subroutine unpack_mytask_args
  end interface
  
  type(c_ptr), value :: cl_arg
  type(c_ptr), value :: buffers

  real(c_float), target  :: multiplier
  type(c_ptr), target :: a_c, b_c
  
  integer, target     :: ma, na, lda, mb, nb, ldb
  real, pointer  :: a(:,:), b(:,:)
  integer :: i
  
  call starpu_f_get_buffer(buffers, 0, c_loc(a_c), c_loc(ma), c_loc(na), c_loc(lda))
  call c_f_pointer(a_c, a,(/lda,na/))

  call starpu_f_get_buffer(buffers, 1, c_loc(b_c), c_loc(mb), c_loc(nb), c_loc(ldb))
  call c_f_pointer(b_c, b,(/ldb,nb/))
  
  call unpack_mytask_args(cl_arg, &
       & c_loc(multiplier))

  
  do i=1, nb
     b(1:min(ma,mb),i) = b(1:min(ma,mb),i) + multiplier*a(1:min(ma,mb),1)
  end do
  call sleep(0.5d0)
  
  write(*,'(i2,"-- mytask :",f7.3)')starpu_f_worker_get_id(), multiplier
  return

contains

  subroutine sleep(t)
    implicit none
    integer :: t_start, t_end, t_rate
    real(kind(1.d0))     :: ta, t
    call system_clock(t_start)
    do
       call system_clock(t_end, t_rate)
       ta = real(t_end-t_start)/real(t_rate)    
       if(ta.gt.t) return
    end do
  end subroutine sleep

  
end subroutine mytask_cpu_func
