#include <starpu.h>
#include <starpu_cuda.h>
#include <starpu_profiling.h>
#include <limits.h>

int starpu_f_init_c(int ncpus) {

  int info;
  struct starpu_conf *conf = malloc(sizeof(struct starpu_conf));
  
  starpu_conf_init(conf);

  if(ncpus > 0)
    conf->ncpus = ncpus;  

  conf->sched_policy_name = "lws";

  info = starpu_init(conf);

  free(conf);
  return info;
}

void starpu_f_get_buffer(void *buffers[], int num, void **A, int *m, int *n, int *lda) {

  *A   = (void *)STARPU_MATRIX_GET_PTR(buffers[num]);
  *m   = (int)STARPU_MATRIX_GET_NX(buffers[num]);
  *n   = (int)STARPU_MATRIX_GET_NY(buffers[num]);
  *lda = (int)STARPU_MATRIX_GET_LD(buffers[num]);

  return;

}


void starpu_f_data_acquire_read(starpu_data_handle_t data) {

  starpu_data_acquire(data, STARPU_R);
  
  return;

}


int starpu_f_sched_ctx_create_c(int *workers, int nworkers, const char *name){
  unsigned ctx;

  int i;
  
  ctx = starpu_sched_ctx_create(workers, nworkers, name, STARPU_SCHED_CTX_POLICY_NAME, "eager", 0);
  
  return (int)ctx;
}


void starpu_f_sched_ctx_set_context_c(int *ctx){

  starpu_sched_ctx_set_context((unsigned*)ctx);

}


void starpu_f_sched_ctx_display_workers_c(int ctx){

  starpu_sched_ctx_display_workers((unsigned)ctx, stderr);

}

void starpu_f_sched_ctx_delete_c(int ctx){

  starpu_sched_ctx_delete((unsigned)ctx);

}

void starpu_f_task_wait_for_all_in_ctx_c(int ctx){

  starpu_task_wait_for_all_in_ctx((unsigned)ctx);

}
